#This example performs a sweep of the selected Lock-In (LI) Frequency Generator.
#It creates an array of frequency values within a certain range, it waits Settling Time,
#and it acquires and averages the selected channels.

import nanonis_spm
import time
import socket
import keyboard
import numpy as np

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Sweep_LI_Index=1          #index of LI Freq. Gen. to sweep
Sweep_LI_Start=1000
Sweep_LI_Stop=10000
Sweep_NrPoints=5
Sweep_LI_Incr=(Sweep_LI_Stop-Sweep_LI_Start)/(Sweep_NrPoints-1)
Sweep_InitSettlingTime=1  #seconds to wait before starting the sweep
Averaging=4               #number of samples to average
Sweep_SettlingTime=1      #seconds to wait after sweeping to each point
Sweep_AcqSignals=[86,87]  #signals to acquire, LI Demod. 1 X&Y
Acq_Signal_1=[]
Acq_Signal_2=[]

#Create array with Sweep LI Frequency values
Sweep_Freq_Values=[]
for point in range(Sweep_NrPoints):
    FreqValue=point*Sweep_LI_Incr+Sweep_LI_Start
    Sweep_Freq_Values.append(FreqValue)
    
#Set LI to Sweep Start value
nanonisInstance.LockIn_ModPhasFreqSet(Sweep_LI_Index, Sweep_LI_Start)

#Wait before starting the sweep
time.sleep(Sweep_InitSettlingTime)

#Sweep the LI Frequency, run the acquisition and averaging of the selected signals
#for as long as the defined number of sweep points
#or until manually stopped (pressing the Spacebar)
AveragedValues=[]
Acq_Finished=False
Stopped_Manually=False
Counter=0
while True:
    AccumulatedValues=np.array([0.0,0.0])
    nanonisInstance.LockIn_ModPhasFreqSet(Sweep_LI_Index,Sweep_Freq_Values[Counter])
    time.sleep(Sweep_SettlingTime)
    for sample in range(Averaging):
        AccumulatedValues[0]=AccumulatedValues[0]+nanonisInstance.Signals_ValsGet(Sweep_AcqSignals,0)[2][1][0][0]
        AccumulatedValues[1]=AccumulatedValues[1]+nanonisInstance.Signals_ValsGet(Sweep_AcqSignals,0)[2][1][1][0]
    AveragedValues.append(AccumulatedValues/Averaging)
    if len(AveragedValues)>=Sweep_NrPoints:
        Acq_Finished=True
        break
    if keyboard.is_pressed(' '):
        Stopped_Manually=True
        break
    Counter=Counter+1

for idx in range(len(AveragedValues)):
    Acq_Signal_1.append(AveragedValues[idx][0])
    Acq_Signal_2.append(AveragedValues[idx][1])
 
print("ACQUIRED SIGNAL 1")
print(Acq_Signal_1)
print("ACQUIRED SIGNAL 2")
print(Acq_Signal_2)

nanonisInstance.close()



#This example acquires one channel (i.e. the tip height) with
#a defined oversampling and logs the oversample value
#in a variable for as many samples as defined or until
#manually stopped (pressing the Spacebar)

import nanonis_spm
import time
import keyboard
import numpy as np
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Acq_Finished=False
Stopped_Manually=False

Channel_Index=30        #tip height
Channel_Oversampling=10
Channel_NrSamples=2000
Channel_Values=[]
Channel_AccumulatedValue=0

#Start the acquisition and oversampling of the selected signal
#for as long as the defined number of samples
#or until manually stopped (pressing the Spacebar)
while True:
    Channel_AccumulatedValue=0
    for sample in range(Channel_Oversampling):
        Channel_AccumulatedValue=Channel_AccumulatedValue+nanonisInstance.Signals_ValGet(Channel_Index,0)[2][0]
    Channel_Values.append(Channel_AccumulatedValue/Channel_Oversampling)
    if len(Channel_Values)>=Channel_NrSamples:
        Acq_Finished=True
        break
    if keyboard.is_pressed(' '):
        Stopped_Manually=True
        break

print("Acquisition finished is {}, Stopped Manually is {}, and the number of acquired samples is {}".format(Acq_Finished,Stopped_Manually,len(Channel_Values)))
print(Channel_Values)

nanonisInstance.close()



#This example performs a Frequency Sweep using the built-in functionality
#in the RF Generator module

import nanonis_spm
import socket
import time
import keyboard

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Limit_Lower=2000000    #Hz
Limit_Higher=10000000  #Hz
Points=10
Mode=1                 #Linear
Dwell=0.25             #s
Repetitions=1
Infinite=2             #Off
OffTime=0              #s
AutoOffTime=1          #On
Direction=0            #Sweep Up

Trigger_Source=1       #Immediate
Trigger_Type=1         #Complete Sweep
Trigger_Mode=1         #Single Shot

#Initialize the frequency to the lower limit of the frequency sweep range
nanonisInstance.APRFGen_FreqSet(0,Limit_Lower)

#Configure the trigger to immediate, single shot
nanonisInstance.APRFGen_TrigPropsSet(0, 0.0001, Trigger_Source, Trigger_Type, 1, Trigger_Mode)

#Switch on the RF Output
nanonisInstance.APRFGen_RFOutOnOffSet(1)

time.sleep(2)

#Configure the limits and properties of the Frequency Sweep mode
nanonisInstance.APRFGen_FreqSwpLimitsSet(Limit_Lower,Limit_Higher)
nanonisInstance.APRFGen_FreqSwpPropsSet(Mode,Dwell,Repetitions,Infinite,Points,OffTime,AutoOffTime)

#Start a Frequency Sweep
nanonisInstance.APRFGen_FreqSwpStart(Direction)

#Rearm the trigger everytime the Enter key is pressed
#Stop the program when pressing the spacebar
while True:
    time.sleep(1)
    if keyboard.is_pressed('enter'):
        nanonisInstance.APRFGen_TrigRearm()
    if keyboard.is_pressed(' '):
        break

nanonisInstance.close()



#This example performs multiple scans,
#increasing the bias voltage by a defined increment after each scan.

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Scan_Number=10
Scan_Autosave=1
Scan_Timeout=50000 #timeout in miliseconds to wait for the scan to finish
Bias_Incr=0.1

#Store the Bias value before scanning
Bias_BeforeScans=nanonisInstance.Bias_Get()[2][0]

#Configure the saving of the images
nanonisInstance.Scan_PropsSet(2,0,Scan_Autosave,"Example_ScanAtDiffBias","no comment",[])

#Run as many scans as defined by Scan_Number and add an increment to Bias after each scan
Bias_Value=Bias_BeforeScans
for scan in range(Scan_Number):
    nanonisInstance.Scan_Action(0,0)
    nanonisInstance.Scan_WaitEndOfScan(5000) #5 seconds timeout to wait for the scan to finish
    Bias_Value=Bias_Value+Bias_Incr
    nanonisInstance.Bias_Set(Bias_Value)
    
time.sleep(1)

#Restore Bias to the original value before scanning
nanonisInstance.Bias_Set(Bias_BeforeScans)

nanonisInstance.close()



#This example applies bias voltage pulses at various Z positions

import nanonis_spm
import time
import numpy as np
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Bias_PulseVolts=1
Bias_PulseWidth=0.1 #seconds
Z_Incr=0.000000001 #meters
Z_NrSteps=10
Z_SettlingTime=0.02 #seconds to wait after setting Z position
    
#Set Tip Lift to 0 in the Z-Controller
nanonisInstance.ZCtrl_TipLiftSet(0)

#Get Z-Ctrl status before pulsing
ZCtrl_BeforeSweep=nanonisInstance.ZCtrl_OnOffGet()[2][0]

#Switch off Z-Ctrl before pulsing
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 10ms (2 seconds timeout)
for WaitTime in range(200):
    time.sleep(0.01)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break

#Start pulsing Bias and set Z position
Z_Pos=nanonisInstance.ZCtrl_ZPosGet()[2][0]
for point in range(Z_NrSteps):
    Z_Pos=Z_Pos+Z_Incr
    nanonisInstance.Bias_Pulse(
    1, #wait until done
    Bias_PulseWidth,
    Bias_PulseVolts,
    np.uint16(1), #Z-Ctrl on hold
    np.uint16(2)  #relative Bias voltage
    )
    nanonisInstance.ZCtrl_ZPosSet(Z_Pos)
    time.sleep(Z_SettlingTime)

#Switch on-off Z-Ctrl according to the status before pulsing
nanonisInstance.ZCtrl_OnOffSet(ZCtrl_BeforeSweep)

nanonisInstance.close()



#This example switches between STM mode and AFM mode

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Switch_To_AFM=1 # to AFM=1, to STM=0
ZController_TipLift=0.00000005 #meters
STM_ControllerName="log Current"
STM_Bias=1 
AFM_ControllerName="Frequency (neg)"
AFM_Bias=0
AFM_ModulatorIndex=1
AFM_ExcitationOut_OnOff=1
AFM_AmplController_OnOff=1
AFM_PhaseController_OnOff=0

#Pause Scan
nanonisInstance.Scan_Action(2,0)
time.sleep (0.1) #wait until X&Y positions are stable

#Set the TipLift in the Z-Controller
nanonisInstance.ZCtrl_TipLiftSet(ZController_TipLift)

#Switch off Z-Ctrl
nanonisInstance.ZCtrl_OnOffSet(0)

#Set the corresponding Z-Controller (STM or AFM) to be the active one
#if the ctrl name is found in the list of existing controls
Applied_ControllerName=""
Applied_ControllerIndex=0
Applied_Bias=0
ControllerNames=nanonisInstance.ZCtrl_CtrlListGet()[2][2]

if Switch_To_AFM==1:
    Applied_ControllerName=AFM_ControllerName
    Applied_Bias=AFM_Bias
else:
    Applied_ControllerName=STM_ControllerName
    Applied_Bias=STM_Bias

CtrlNameFound=False
CtrlName=""
for CtrlName in ControllerNames:
    if CtrlName==Applied_ControllerName:
        CtrlNameFound=True
        break
    else:
        Applied_ControllerIndex=Applied_ControllerIndex+1

if CtrlNameFound is True:
    nanonisInstance.ZCtrl_ActiveCtrlSet(Applied_ControllerIndex)

    #Set Bias to the corresponding value (STM or AFM) 
    nanonisInstance.Bias_Set(Applied_Bias)

    #Switch the excitation output
    if Switch_To_AFM==1:
        nanonisInstance.PLL_OutOnOffSet(AFM_ModulatorIndex,AFM_ExcitationOut_OnOff)

    time.sleep(0.05)

    #Switch the Amplitude Ctrl
    if Switch_To_AFM==1:
        nanonisInstance.PLL_AmpCtrlOnOffSet(AFM_ModulatorIndex,AFM_AmplController_OnOff)

    #Switch the Phase Ctrl
    if Switch_To_AFM==1:
        nanonisInstance.PLL_PhasCtrlOnOffSet(AFM_ModulatorIndex,AFM_PhaseController_OnOff)
        
    time.sleep(0.1)

    #Switch on-off Z-Ctrl
    nanonisInstance.ZCtrl_OnOffSet(1)
else:
    print ("The controller name can't be found in the list of available controllers")

nanonisInstance.close()



#Automatic approach implemented using Nanonis Programming Interface functions.
#This can be used if you want to execute special functions after each 
#motor step or if you want to control your motor remotely.

import nanonis_spm
import time
import keyboard
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Motor_NrSteps=10 #number of steps to move toward the sample

#Fully withdraw the tip
nanonisInstance.ZCtrl_Withdraw(1,-1) #Wait until done

#Wait a bit after withdrawing the tip
time.sleep(0.2) #seconds
    
#Read the upper piezo range limit
Piezo_RangeLimit_Upper=nanonisInstance.ZCtrl_ZPosGet()[2][0]

#Calculate the lower piezo range limit
Piezo_RangeLimit_Lower=Piezo_RangeLimit_Upper*(-0.95)

#Start the auto-approach procedure
while True:
    #Switch on Z-Ctrl
    nanonisInstance.ZCtrl_OnOffSet(1)

    #Wait until Z-Ctrl is really switched on
    #checking the status every 10ms (2 seconds timeout)
    for WaitTime in range(200):
        time.sleep(0.01)
        if nanonisInstance.ZCtrl_OnOffGet()[2][0]==1:
            break

    #Read the Z position repeatedly until:
    #- surface reached=> new_Z >= previous_Z (controlled position)
    #- or Spacebar button is pressed to manually stop the procedure
    #- or the lower limit is reached
    Piezo_Position=float('inf')
    Surface_Reached=False
    Stopped_Manually=False
    while True:
        time.sleep(0.15)
        Piezo_Position_New=nanonisInstance.ZCtrl_ZPosGet()[2][0]
        if Piezo_Position_New>=Piezo_Position:
            Surface_Reached=True
            break
        if keyboard.is_pressed(' '):
            Stopped_Manually=True
            break
        if Piezo_Position_New<Piezo_RangeLimit_Lower:
            break
        else:
            Piezo_Position=Piezo_Position_New
            
    print("Surface_Reached is {} and Stopped Manually is {}".format(Surface_Reached,Stopped_Manually))
        
    #If surface reached or procedure manually stopped:
    #- withdraw the tip
    #- move the motor few steps toward the sample
    #- wait motor's end-of-move
    if Surface_Reached==True or Stopped_Manually==True:
        break
    else:    
        nanonisInstance.ZCtrl_Withdraw(1,-1) #Wait until done
        time.sleep(0.2) #seconds
        nanonisInstance.Motor_StartMove(5,Motor_NrSteps,0,1) #move in Z- direction and wait for it to finish

nanonisInstance.close()




#This example performs a custom Z sweep.
#The selected channels are acquired at each Z sweep point.

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Sweep_Distance=0.0000005
Sweep_NrPoints=10
Sweep_Incr=Sweep_Distance/Sweep_NrPoints
Sweep_SettlingTime=0.2 #seconds to wait after sweeping to a point
Sweep_AcqSignals=[0,30] #signals to acquire, Current & Z
AcqSignal_Current=[]
AcqSignal_Z=[]

#Get Z-Ctrl status before sweeping
ZCtrl_BeforeSweep=nanonisInstance.ZCtrl_OnOffGet()[2][0]

#Switch off Z-Ctrl before sweeping
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 100ms (2 seconds timeout)
for WaitTime in range(20):
    time.sleep(0.1)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break

#Get current Z position
Sweep_Value=nanonisInstance.ZCtrl_SetpntGet()[2][0]

#Sweep Z & acquire signals
for point in range(Sweep_NrPoints-1):
    Sweep_Value=Sweep_Value+Sweep_Incr
    nanonisInstance.ZCtrl_ZPosSet(Sweep_Value)
    time.sleep(Sweep_SettlingTime)
    AcqSignal_Current.append(nanonisInstance.Signals_ValsGet(Sweep_AcqSignals, True)[2][1][0])
    AcqSignal_Z.append(nanonisInstance.Signals_ValsGet(Sweep_AcqSignals, True)[2][1][1])

#Switch on-off Z-Ctrl according to the status before the sweep
nanonisInstance.ZCtrl_OnOffSet(ZCtrl_BeforeSweep)

print("CURRENT")
print(AcqSignal_Current)
print("Z")
print(AcqSignal_Z)

nanonisInstance.close()



#This example performs a custom bias sweep. You can create an array of
#Bias voltages and it will record the selected channels at each Bias voltage.

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Sweep_StartBias=1
Sweep_StopBias=2
Sweep_NrPoints=10
Sweep_BiasIncr=(Sweep_StopBias-Sweep_StartBias)/(Sweep_NrPoints-1)
Sweep_SettlingTime=0.2 #seconds to wait after sweeping to a point
Sweep_AcqSignals=[0,24] #signals to acquire, Current & Bias
AcqSignal_Current=[]
AcqSignal_Bias=[]

#Create array with Sweep Bias values
Sweep_BiasValues=[]
for point in range(Sweep_NrPoints):
    BiasValue=point*Sweep_BiasIncr+Sweep_StartBias
    Sweep_BiasValues.append(BiasValue)
    
#Set Bias to Sweep Start value
nanonisInstance.Bias_Set(Sweep_StartBias)

#Get Z-Ctrl status before sweeping
ZCtrl_BeforeSweep=nanonisInstance.ZCtrl_OnOffGet()[2][0]

#Switch off Z-Ctrl before sweeping
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 100ms (2 seconds timeout)
for WaitTime in range(20):
    time.sleep(0.1)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break
    
#Sweep Bias & acquire signals
for point in range(Sweep_NrPoints):
    nanonisInstance.Bias_Set(Sweep_BiasValues[point])
    time.sleep(Sweep_SettlingTime)
    AcqSignal_Current.append(nanonisInstance.Signals_ValsGet(Sweep_AcqSignals, True)[2][1][0])
    AcqSignal_Bias.append(nanonisInstance.Signals_ValsGet(Sweep_AcqSignals, True)[2][1][1])


#Switch on-off Z-Ctrl according to the status before the sweep
nanonisInstance.ZCtrl_OnOffSet(ZCtrl_BeforeSweep)

print("CURRENT")
print(AcqSignal_Current)
print("BIAS")
print(AcqSignal_Bias)

nanonisInstance.close()



#This example performs a 2D sweep using a User output and the Generic Sweeper.
#For each value of the User output (y axis)
#it will perform a Generic Sweep (x axis). 

import nanonis_spm
import numpy
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

#Configuration variables of the Generic Sweeper
#The acquired signals are configured in the Generic Sweeper module for this example
Sweep_X_Start=1
Sweep_X_Stop=2
Sweep_X_NrPoints=10
Sweep_X_InitSettling=300 #miliseconds
Sweep_X_Settling=200 #miliseconds
Sweep_X_Period=100 #miliseconds
Sweep_X_MaxSlewRate=20 #units/second
Sweep_X_Autosave=0
Sweep_X_ShowSaveDialog=0
Sweep_X_Basename=""

#Configuration variables of the 2nd dimension
Sweep_Y_UserOutputIndex=3
Sweep_Y_Start=3
Sweep_Y_Stop=4
Sweep_Y_NrPoints=2
Sweep_Y_Incr=(Sweep_Y_Stop-Sweep_Y_Start)/(Sweep_Y_NrPoints-1)

#Create array with Sweep values for the 2nd dimension
Sweep_Y_Values=[]
for point in range(Sweep_Y_NrPoints):
    Y_Value=point*Sweep_Y_Incr+Sweep_Y_Start
    Sweep_Y_Values.append(Y_Value)
    
#Configure the Generic Sweeper properties
nanonisInstance.GenSwp_PropsSet(
    Sweep_X_InitSettling,
    Sweep_X_MaxSlewRate,
    Sweep_X_NrPoints,
    Sweep_X_Period,
    Sweep_X_Autosave,
    Sweep_X_ShowSaveDialog,
    Sweep_X_Settling
    )

#Configure the Generic Sweeper limits
nanonisInstance.GenSwp_LimitsSet(Sweep_X_Start,Sweep_X_Stop)

#Run the 2D Sweep & acquire signals
GenSweeper_Data_ForOneYValue=[]
GenSweeper_NrAcqChannels=nanonisInstance.GenSwp_AcqChsGet()[2][0]+1 #the swept channel is part of the acquired data
GenSweeper_Data_Total=[[]]*GenSweeper_NrAcqChannels

for point in range(Sweep_Y_NrPoints):
    #Set the user output value configured for the 2nd dimension
    nanonisInstance.UserOut_ValSet(Sweep_Y_UserOutputIndex,Sweep_Y_Values[point])
    #Run the Generic Sweeper
    GenSweeper_Data_ForOneYValue=nanonisInstance.GenSwp_Start(1,1,Sweep_X_Basename,0,1)[2]
    for SignalIdx in range(GenSweeper_NrAcqChannels):
        GenSweeper_Data_Total[SignalIdx]=GenSweeper_Data_Total[SignalIdx]+GenSweeper_Data_ForOneYValue[5][SignalIdx].tolist()
        
print(GenSweeper_Data_Total)

nanonisInstance.close()



#Moves the tip to a specified location and performs Bias pulses there
#Useful to try to clean the tip
#without affecting the sample at the current position


import nanonis_spm
import time
import numpy as np
import keyboard
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

XY_GoToPos=[-0.0000000013,0.0000000035] #meters
XY_StayAtCurrentPos=False
BiasPulse_Volts=0.5
BiasPulse_Width=0.2 #seconds
BiasPulse_ZCtrl=1 #hold the Z-Ctrl
BiasPulse_NrPulses=5
BiasPulse_Pause=0.2 #seconds to wait between pulses
    
#Return the tip position
XY_CurrentPos=nanonisInstance.FolMe_XYPosGet(1)[2]

#If clean the tip at the current position, skip moving the tip
if XY_StayAtCurrentPos==False:
    nanonisInstance.FolMe_XYPosSet(XY_GoToPos[0],XY_GoToPos[1],1)

#Start pulsing Bias
for pulse in range(BiasPulse_NrPulses):
    nanonisInstance.Bias_Pulse(
    1, #wait until done
    BiasPulse_Width,
    BiasPulse_Volts,
    np.uint16(BiasPulse_ZCtrl), #Z-Ctrl on hold
    np.uint16(1)  #absolute Bias voltage
    )
    time.sleep(BiasPulse_Pause)
    #Stop pulsing if the Spacebar button is pressed
    if keyboard.is_pressed(' '): 
        break

#Move to the original position
if XY_StayAtCurrentPos==False:
    nanonisInstance.FolMe_XYPosSet(XY_CurrentPos[0],XY_CurrentPos[1],1)

nanonisInstance.close()



#This example records the Z channel (i.e. the tip height) and 
#calculates the drift along this axis.
#The drift speed can then be applied to compensate for it.
#Before starting the measurement, the tip must be in contact with the sample
#and the Z-Controller must be running.

import nanonis_spm
import time
import keyboard
import numpy as np
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Z_Drift_Duration=1 #the time the Z drift measurement is running (in minutes)
Z_Drift_Duration_Seconds=Z_Drift_Duration*60
Z_Drift_IterationTime=0.1 #how often we read the Z position to calculate drift (in seconds)
Z_Drift_Apply=True #if True, the Z drift is applied after the Z drift measurement

Z_Drift_Value=0 #meters/second
Z_Drift_Comp_Vz=0
Z_Drift_Finished=False
Stopped_Manually=False
Z_Drift_Lst=[]
Z_Pos_Lst=[]
Time_Lst=[]
Time=0


#Start the Z drift measurement by reading the Z position
#for as long as the Z_Drift_Duration time
#or until manually stopped (pressing the Spacebar)
while True:
    time.sleep(Z_Drift_IterationTime)
    Time=Time+Z_Drift_IterationTime
    Time_Lst.append(Time)
    Z_Pos_Lst.append(nanonisInstance.ZCtrl_ZPosGet()[2][0])
    #Calculate the linear fit and return the slope
    Z_Drift_Value=np.linalg.lstsq(np.vstack([np.array(Time_Lst),np.ones(len(np.array(Time_Lst)))]).T, np.array(Z_Pos_Lst), rcond=None)[0]
    Z_Drift_Lst.append(Z_Drift_Value[0])
    if Time>=Z_Drift_Duration_Seconds:
        Z_Drift_Finished=True
        break
    if keyboard.is_pressed(' '):
        Stopped_Manually=True
        break

#if Z_Drift_Apply is True, apply the calculated Z Drift to compensate for it
if Z_Drift_Apply==True:
    Z_Drift_Comp=nanonisInstance.Piezo_DriftCompGet()[2]
    Z_Drift_Comp_OnOff=Z_Drift_Comp[0] #Drift compensation status (on or off)
    if Z_Drift_Comp_OnOff==True:
        Z_Drift_Comp_Vz=Z_Drift_Comp[3]+Z_Drift_Value[0] #Current linear speed applied to the Z piezo
    else:
        Z_Drift_Comp_Vz=Z_Drift_Value[0]
    nanonisInstance.Piezo_DriftCompSet(1,Z_Drift_Comp[1],Z_Drift_Comp[2],Z_Drift_Comp_Vz,Z_Drift_Comp[7])


print("Z drift measurement finished is {}, Stopped Manually is {}, and the Z drift is {} m/s".format(Z_Drift_Finished,Stopped_Manually,Z_Drift_Comp_Vz))

nanonisInstance.close()



#This procedure is mostly used to shape a STM tip,
#dipping it into the sample and pulling it back slowly
#First ramp Z (usually to approach the tip), then apply a bias voltage, and
#finally ramp Z again (usually to withdraw the tip)

import nanonis_spm
import time
import numpy as np
import keyboard
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

BeforeRamp_ZCrl_SwitchOffDelay=0.2 #seconds
BeforeRamp_CurrentGain_Apply=True
BeforeRamp_CurrentGain_Index=0
BeforeRamp_Bias_Apply=True
BeforeRamp_Bias_Volts=1 
Ramp1_TipLift=-0.000000002 #meters
Ramp1_LiftTime=0.2 #seconds
Bias_Volts=2
Bias_SettlingTime=0.1 #seconds
Ramp2_TipLift=0.000000005 #meters
Ramp2_LiftTime=0.1 #seconds
AfterRamp_ZCrl_Restore=True
AfterRamp_WaitTime=0.1 #seconds

#Remember Z-Ctrl status, Bias, and Current gain before ramping
Original_ZCtrl=nanonisInstance.ZCtrl_OnOffGet()[2][0]
Original_CurrentGain_Index=nanonisInstance.Current_GainsGet()[2][3]
Original_Bias=nanonisInstance.Bias_Get()[2][0]

#Don't use built-in TipLift as it doesn't ramp
#and set Switch Off Delay
nanonisInstance.ZCtrl_TipLiftSet(0)
nanonisInstance.ZCtrl_SwitchOffDelaySet(BeforeRamp_ZCrl_SwitchOffDelay)

#Switch off Z-Ctrl before ramping
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 100ms (2 seconds timeout)
for WaitTime in range(20):
    time.sleep(0.1)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break

#Apply the optional parameters
if BeforeRamp_CurrentGain_Apply==True:
    nanonisInstance.Current_GainSet(BeforeRamp_CurrentGain_Index,-1)
if BeforeRamp_Bias_Apply==True:
    nanonisInstance.Bias_Set(BeforeRamp_Bias_Volts)
    time.sleep(Bias_SettlingTime)

#Execute first Z ramp
Ramp1_Z=nanonisInstance.Signals_ValGet(30,True)[2][0]
Ramp1_Z_Comparison=(Ramp1_Z+Ramp1_TipLift)>Ramp1_Z
Ramp1_iterations=round(Ramp1_LiftTime/0.005)
Ramp1_Z_Incr=(Ramp1_Z+Ramp1_TipLift)/Ramp1_iterations
Ramp1_Z_Original=Ramp1_Z+Ramp1_TipLift

for i in range(Ramp1_iterations):
    time.sleep(0.005) #every 5 ms
    if i+1==Ramp1_iterations:
        break
    else:
        Ramp1_Z=Ramp1_Z+Ramp1_TipLift
    nanonisInstance.ZCtrl_ZPosSet(Ramp1_Z)

#Apply Bias   
nanonisInstance.Bias_Set(Bias_Volts)
time.sleep(Bias_SettlingTime)

#Execute second Z ramp
Ramp2_Z=nanonisInstance.Signals_ValGet(30,True)[2][0]
Ramp2_Z_Comparison=(Ramp2_Z+Ramp2_TipLift)>Ramp2_Z
Ramp2_iterations=round(Ramp2_LiftTime/0.005)
Ramp2_Z_Incr=(Ramp2_Z+Ramp2_TipLift)/Ramp2_iterations
Ramp2_Z_Original=Ramp2_Z+Ramp2_TipLift

for i in range(Ramp2_iterations):
    time.sleep(0.005) #every 5 ms
    if i+1==Ramp2_iterations:
        break
    else:
        Ramp2_Z=Ramp2_Z+Ramp2_TipLift
    nanonisInstance.ZCtrl_ZPosSet(Ramp2_Z)
    
#Restore parameters
if BeforeRamp_Bias_Apply==True:
    nanonisInstance.Bias_Set(Original_Bias)
    time.sleep(Bias_SettlingTime)
if BeforeRamp_CurrentGain_Apply==True:
    nanonisInstance.Current_GainSet(Original_CurrentGain_Index,-1)
time.sleep(AfterRamp_WaitTime)
if AfterRamp_ZCrl_Restore==True:
    nanonisInstance.ZCtrl_OnOffSet(Original_ZCtrl)

nanonisInstance.close()



#This example performs a bias spectroscopy consisting of several user-definable segments
#using the Bias Spectroscopy module.

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

Spectr_Channels=[0,24]  #Acquired channels: Current and Bias
Spectr_StartBias=[-1,-0.2,0.2,1.2]
Spectr_StopBias=[-0.2,0.2,1,2]
Spectr_NrPoints=[50,400,50,100]
ZCtrl_TipLift=0         #meters
ZCtrl_SwitchOffDelay=2  #seconds
AcqSignal_Bias=[]
AcqSignal_Current=[]

#Open the Bias Spectroscopy module
nanonisInstance.BiasSpectr_Open()

#Store the current Bias value and Spectroscopy timings
Bias_BeforeSpectr=nanonisInstance.Bias_Get()[2][0]
SpectrTimings_BeforeSpectr=nanonisInstance.BiasSpectr_TimingGet()[2]

#Configure the Bias Spectroscopy module
nanonisInstance.BiasSpectr_ChsSet(Spectr_Channels)
nanonisInstance.BiasSpectr_TimingSet(0,0,float("NaN"),float("NaN"),float("NaN"),float("NaN"),0,0) #NaN means no change
nanonisInstance.BiasSpectr_AdvPropsSet(2,0,0,0)

#Get Z-Ctrl status before sweeping
ZCtrl_BeforeSpectr=nanonisInstance.ZCtrl_OnOffGet()[2][0]

#Set Z-Controller Tip Lift & Switch Off delay before switching it off
nanonisInstance.ZCtrl_TipLiftSet(ZCtrl_TipLift)
nanonisInstance.ZCtrl_SwitchOffDelaySet(ZCtrl_SwitchOffDelay)
time.sleep(0.1)

#Switch off Z-Ctrl before running the spectroscopy
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 200ms (8 seconds timeout)
for WaitTime in range(40):
    time.sleep(0.2)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break

#Run Bias Spectroscopy as many times as defined segments
SegmentIndex=0
for SegmentIndex in range(len(Spectr_StartBias)):
    nanonisInstance.BiasSpectr_PropsSet(
        0,0,2,Spectr_NrPoints[SegmentIndex],float("NaN"),1,2) #set to autosave, no dialog window
    nanonisInstance.BiasSpectr_LimitsSet(                     #set the segment limits
        Spectr_StartBias[SegmentIndex],Spectr_StopBias[SegmentIndex])
    Spectr_AcquiredData=nanonisInstance.BiasSpectr_Start(1,"Example_BiasSpectroscopy_MultiSegment")[2][5]
    AcqSignal_Current=AcqSignal_Current+Spectr_AcquiredData[1].tolist()
    AcqSignal_Bias=AcqSignal_Bias+Spectr_AcquiredData[2].tolist()
    time.sleep(0.2)

#Restore Bias and Spectroscopy timings
nanonisInstance.Bias_Set(Bias_BeforeSpectr)
nanonisInstance.BiasSpectr_TimingSet(
    SpectrTimings_BeforeSpectr[0],
    SpectrTimings_BeforeSpectr[1],
    SpectrTimings_BeforeSpectr[2],
    SpectrTimings_BeforeSpectr[3],
    SpectrTimings_BeforeSpectr[4],
    SpectrTimings_BeforeSpectr[5],
    SpectrTimings_BeforeSpectr[6],
    SpectrTimings_BeforeSpectr[7])
time.sleep(0.1)

#Switch on-off Z-Ctrl according to the status before the sweep
nanonisInstance.ZCtrl_OnOffSet(ZCtrl_BeforeSpectr)

print("CURRENT")
print(AcqSignal_Current)
print("BIAS")
print(AcqSignal_Bias)

nanonisInstance.close()



#This example perfoms force-distance measurement with offset compensation.
#It requires the PLL module to run and
#setting up the Z-Spectroscopy configuration

import nanonis_spm
import time
import socket

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

ZCtrl_TipLift=0
ZCtrl_SettlingTime=0.05 #seconds to wait after switching off the Z-Ctrl
    
#Set Tip Lift to 0 in the Z-Controller
nanonisInstance.ZCtrl_TipLiftSet(ZCtrl_TipLift)

#Get Z-Ctrl status
ZCtrl_BeforeSweep=nanonisInstance.ZCtrl_OnOffGet()[2][0]

#Switch off Z-Ctrl
nanonisInstance.ZCtrl_OnOffSet(0)

#Wait until Z-Ctrl is really switched off
#checking the status every 10ms (2 seconds timeout)
for WaitTime in range(200):
    time.sleep(0.01)
    if nanonisInstance.ZCtrl_OnOffGet()[2][0]==0:
        break

#Wait after switching off the Z-Ctrl
time.sleep(ZCtrl_SettlingTime)

#Auto-center the frequency shift in the PLL module
nanonisInstance.PLL_FreqShiftAutoCenter(0)

#Run the Z-Spectroscopy measurement
nanonisInstance.ZSpectr_Start(0,"")

#Switch on-off Z-Ctrl according to the status before pulsing
nanonisInstance.ZCtrl_OnOffSet(ZCtrl_BeforeSweep)

nanonisInstance.close()



#This example performs multiple Power Sweeps by using the Generic Sweeper
#to sweep the power in the RF Generator module multiple times with different limits

import nanonis_spm
import numpy
import socket
import time

connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
connection.connect(("localhost",6501))

nanonisInstance = nanonis_spm.Nanonis(connection)
nanonisInstance.returnDebugInfo(1)

#Generic Sweeper variables
Sweep_Start=[1,2,-1,-4]  #Sweep start values for the different sweeps to perfom 
Sweep_Stop=[5,3,2,1]   #(the list size determines the number of sweeps)
Sweep_NrPoints=[10,5,10,5]

#Generic Sweeper properties common to all sweeps
Sweep_InitSettling=300 #miliseconds
Sweep_Settling=200 #miliseconds
Sweep_Period=100 #miliseconds
Sweep_MaxSlewRate=20 #units/second
Sweep_Autosave=0
Sweep_ShowSaveDialog=0
Sweep_Basename=""

#Select the power signal as the signal to sweep
nanonisInstance.GenSwp_SwpSignalSet("Power (dBm)")

#Initialize the variables storing the acquired data
GenSweeper_Data_OneSweep=[]
GenSweeper_NrAcqChannels=nanonisInstance.GenSwp_AcqChsGet()[2][0]+1 #the swept channel is part of the acquired data
GenSweeper_Data_Total=[[]]*GenSweeper_NrAcqChannels

#Run the sweeps
for SweepIndex in range(len(Sweep_Start)):
    #Configure the Generic Sweeper properties
    nanonisInstance.GenSwp_PropsSet(
        Sweep_InitSettling,
        Sweep_MaxSlewRate,
        Sweep_NrPoints[SweepIndex],
        Sweep_Period,
        Sweep_Autosave,
        Sweep_ShowSaveDialog,
        Sweep_Settling)
    #Configure the Generic Sweeper limits
    nanonisInstance.GenSwp_LimitsSet(Sweep_Start[SweepIndex],Sweep_Stop[SweepIndex])
    #Run the Generic Sweeper
    GenSweeper_Data_OneSweep=nanonisInstance.GenSwp_Start(1,1,Sweep_Basename,0,0)[2]
    for SignalIdx in range(GenSweeper_NrAcqChannels):
        GenSweeper_Data_Total[SignalIdx]=GenSweeper_Data_Total[SignalIdx]+GenSweeper_Data_OneSweep[5][SignalIdx].tolist()
        
print(GenSweeper_Data_Total)

nanonisInstance.close()

